#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>

#define ERROR_OUTPUT stderr
#define ASSERT(b) {if (!(b)) {fprintf(ERROR_OUTPUT, "ASSERT failed on line %d, file %s.\n", __LINE__, __FILE__); exit(1);}}

typedef struct _PointT {
  int index; // the index of this point in the dataset list of points
  float *coordinates;
  int tag;
} PointT, *PPointT;

typedef struct _ERRINDEX{
	float data1;
	float data2;
	float error;
	int index;
}ERRINDEX, *PERRINDEX;

// Generate a random real distributed uniformly in [rangeStart,
// rangeEnd]. Input must satisfy: rangeStart <= rangeEnd. The
// granularity of generated random reals is given by RAND_MAX.
float genUniformRandom(float rangeStart, float rangeEnd){
  ASSERT(rangeStart <= rangeEnd);
  float r;
  r = rangeStart + ((rangeEnd - rangeStart) * (float)random() / (float)RAND_MAX);
  ASSERT(r >= rangeStart && r <= rangeEnd);
  return r;
}

// Generate a random 32-bits unsigned (int) in the range
// [rangeStart, rangeEnd]. Inputs must satisfy: rangeStart <=
// rangeEnd.
int genRandomUns32(int rangeStart, int rangeEnd){
  ASSERT(rangeStart <= rangeEnd);
  int r;
  if (RAND_MAX >= rangeEnd - rangeStart) {
    r = rangeStart + (int)((rangeEnd - rangeStart + 1.0) * random() / (RAND_MAX + 1.0));
  } else {
    r = rangeStart + (int)((rangeEnd - rangeStart + 1.0) * ((long int)random() * ((long int)RAND_MAX + 1) + (long int)random()) / ((long int)RAND_MAX * ((long int)RAND_MAX + 1) + (long int)RAND_MAX + 1.0));
  }
  ASSERT(r >= rangeStart && r <= rangeEnd);
  return r;
}

void usage(char *programName){
  printf("Usage: %s kde_from_allpts kde_from_sample #pts_in_test_file k\n", programName);
}

void readDataSetFromFile(char *filename, float* dataSetPoints, int d,int nPoints){
  FILE *f = fopen(filename, "rt");
  for(int i = 0; i < nPoints; i++){
    for(int j = 0; j < d; j++){
         int dummy = fscanf(f, "%f", &(dataSetPoints[i*d+j]));
    }
  }
  fclose(f);
}

int main(int nargs, char **args){


	if(nargs < 4){
        usage(args[0]);
		exit(1);
    }

	srand(time(0));
	FILE* fpin1 = fopen(args[1],"r");
	FILE* fpin2 = fopen(args[2],"r");
	float *pTargets;
	
	int nPoints = atoi(args[4]);

	int dim = atoi(args[5]);
	int k = atoi(args[6]);

	float data1 = 0;
	float data2 = 0;
	char str [80];
	float max = 0;
	float diff = 0;

	pTargets = new float[nPoints*dim];
	readDataSetFromFile(args[3],pTargets,dim,nPoints);

	PERRINDEX errIndex = new ERRINDEX[k];
	for(int i = 0; i<nPoints; i++){
		int dummy = fscanf(fpin1, "%f", &(data1));
		int dummy2 = fscanf(fpin2, "%f", &(data2));
		diff = fabs(data1-data2);///((data1>data2)?data1:data2);

		//diff is bigger than the largest error so far
		//printf("diff = %f\n",diff);
		if(diff > errIndex[0].error){
			for(int m = k-1; m >= 1; m--){
            	errIndex[m] =errIndex[m-1];
            }
			errIndex[0].error = diff;
            errIndex[0].data1 = data1;
			errIndex[0].data2 = data2;
			errIndex[0].index = i;
		}else if(diff > errIndex[k-1].error){
			for(int j = k-2;  j >= 0; j--){
				if(diff < errIndex[j].error){
					//printf("j = %d\n",j);
					for(int m = k-1; m >= j+2; m--){
						errIndex[m] =errIndex[m-1];
					}
					errIndex[j+1].error = diff;
					errIndex[j+1].data1 = data1;
					errIndex[j+1].data2 = data2;
					errIndex[j+1].index = i;
					break;
				}
			}
		}
		//if diff is less than errIndex[k-1].error, we don't need to care about it
		
		//printf("data1 = %f, data2 = %f, diff = %f, max = %f\n",data1, data2, diff, max);
		if(diff > max)	max = diff;
		/*printf("i = %d\n",i);
		for(int i = 0 ; i < k; i++){
    	    printf("%f, %f, error = %f\n",errIndex[i].data1, errIndex[i].data2, errIndex[i].error);
	    }*/

	}
	printf("%f\n",max);
	//for(int i = 0 ; i < k; i++){
		//printf("index = %d, %f, %f, error = %f\n",errIndex[i].index, pTargets[errIndex[i].index*dim], pTargets[errIndex[i].index*dim+1], errIndex[i].error);
	//	printf("%f %f %f\n",errIndex[i].error, errIndex[i].data1, errIndex[i].data2); 
	//}
	delete []errIndex;
	fclose(fpin1);
	fclose(fpin2);

}
